@@ -175,6 +175,12 @@ public class NetworkVO implements Network {
     @Column(name = "external_id")
     String externalId;
 
+    @Transient
+    String routerIp;
+
+    @Transient
+    String routerIpv6;
+
     @Transient
     transient String vlanIdAsUUID;
 
@@ -672,4 +678,20 @@ public class NetworkVO implements Network {
     public void setPvlanType(PVlanType pvlanType) {
         this.pVlanType = pvlanType;
     }
+
+    public String getRouterIp() {
+        return routerIp;
+    }
+
+    public void setRouterIp(String routerIp) {
+        this.routerIp = routerIp;
+    }
+
+    public String getRouterIpv6() {
+        return routerIpv6;
+    }
+
+    public void setRouterIpv6(String routerIpv6) {
+        this.routerIpv6 = routerIpv6;
+    }
 }
