@@ -17,6 +17,7 @@
 
 package com.cloud.kubernetes.cluster.actionworkers;
 
+import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.Field;
 import java.util.ArrayList;
@@ -47,7 +48,7 @@ import com.cloud.exception.InsufficientCapacityException;
 import com.cloud.exception.InsufficientServerCapacityException;
 import com.cloud.exception.ManagementServerException;
 import com.cloud.exception.NetworkRuleConflictException;
-import com.cloud.exception.ResourceAllocationException;
+import com.cloud.exception.OperationTimedoutException;
 import com.cloud.exception.ResourceUnavailableException;
 import com.cloud.host.Host;
 import com.cloud.host.HostVO;
@@ -56,6 +57,7 @@ import com.cloud.hypervisor.Hypervisor;
 import com.cloud.kubernetes.cluster.KubernetesCluster;
 import com.cloud.kubernetes.cluster.KubernetesClusterDetailsVO;
 import com.cloud.kubernetes.cluster.KubernetesClusterManagerImpl;
+import com.cloud.kubernetes.cluster.KubernetesClusterVO;
 import com.cloud.kubernetes.cluster.utils.KubernetesClusterUtil;
 import com.cloud.network.IpAddress;
 import com.cloud.network.Network;
@@ -72,8 +74,9 @@ import com.cloud.network.rules.dao.PortForwardingRulesDao;
 import com.cloud.offering.ServiceOffering;
 import com.cloud.resource.ResourceManager;
 import com.cloud.storage.Volume;
-import com.cloud.storage.VolumeApiService;
 import com.cloud.storage.VolumeVO;
+import com.cloud.storage.dao.LaunchPermissionDao;
+import com.cloud.storage.VolumeApiService;
 import com.cloud.storage.dao.VolumeDao;
 import com.cloud.user.Account;
 import com.cloud.user.SSHKeyPairVO;
@@ -82,11 +85,13 @@ import com.cloud.utils.Pair;
 import com.cloud.utils.StringUtils;
 import com.cloud.utils.component.ComponentContext;
 import com.cloud.utils.db.Transaction;
+import com.cloud.utils.db.TransactionCallback;
 import com.cloud.utils.db.TransactionCallbackWithException;
 import com.cloud.utils.db.TransactionStatus;
-import com.cloud.utils.exception.ExecutionException;
+import com.cloud.utils.exception.CloudRuntimeException;
 import com.cloud.utils.net.Ip;
 import com.cloud.utils.net.NetUtils;
+import com.cloud.utils.ssh.SshHelper;
 import com.cloud.vm.Nic;
 import com.cloud.vm.UserVmManager;
 import com.cloud.vm.VirtualMachine;
@@ -124,6 +129,8 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
     @Inject
     protected UserVmManager userVmManager;
     @Inject
+    protected LaunchPermissionDao launchPermissionDao;
+    @Inject
     protected VolumeApiService volumeService;
     @Inject
     protected VolumeDao volumeDao;
@@ -142,15 +149,15 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
     private String getKubernetesNodeConfig(final String joinIp, final boolean ejectIso) throws IOException {
         String k8sNodeConfig = readResourceFile("/conf/k8s-node.yml");
         final String sshPubKey = "{{ k8s.ssh.pub.key }}";
-        final String joinIpKey = "{{ k8s_master.join_ip }}";
-        final String clusterTokenKey = "{{ k8s_master.cluster.token }}";
+        final String joinIpKey = "{{ k8s_control_node.join_ip }}";
+        final String clusterTokenKey = "{{ k8s_control_node.cluster.token }}";
         final String ejectIsoKey = "{{ k8s.eject.iso }}";
         String pubKey = "- \"" + configurationDao.getValue("ssh.publickey") + "\"";
         String sshKeyPair = kubernetesCluster.getKeyPair();
         if (!Strings.isNullOrEmpty(sshKeyPair)) {
             SSHKeyPairVO sshkp = sshKeyPairDao.findByName(owner.getAccountId(), owner.getDomainId(), sshKeyPair);
             if (sshkp != null) {
-                pubKey += "\n  - \"" + sshkp.getPublicKey() + "\"";
+                pubKey += "\n      - \"" + sshkp.getPublicKey() + "\"";
             }
         }
         k8sNodeConfig = k8sNodeConfig.replace(sshPubKey, pubKey);
@@ -181,7 +188,7 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         if (!Strings.isNullOrEmpty(dockerUserName) && !Strings.isNullOrEmpty(dockerPassword)) {
             // do write file for  /.docker/config.json through the code instead of k8s-node.yml as we can no make a section
             // optional or conditionally applied
-            String dockerConfigString = "write-files:\n" +
+            String dockerConfigString = "write_files:\n" +
                     "  - path: /.docker/config.json\n" +
                     "    owner: core:core\n" +
                     "    permissions: '0644'\n" +
@@ -194,7 +201,7 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
                     "          }\n" +
                     "         }\n" +
                     "      }";
-            k8sNodeConfig = k8sNodeConfig.replace("write-files:", dockerConfigString);
+            k8sNodeConfig = k8sNodeConfig.replace("write_files:", dockerConfigString);
             final String dockerUrlKey = "{{docker.url}}";
             final String dockerAuthKey = "{{docker.secret}}";
             final String dockerEmailKey = "{{docker.email}}";
@@ -307,12 +314,11 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
             Field f = startVm.getClass().getDeclaredField("id");
             f.setAccessible(true);
             f.set(startVm, vm.getId());
-            userVmService.startVirtualMachine(startVm);
+            itMgr.advanceStart(vm.getUuid(), null, null);
             if (LOGGER.isInfoEnabled()) {
                 LOGGER.info(String.format("Started VM : %s in the Kubernetes cluster : %s", vm.getDisplayName(), kubernetesCluster.getName()));
             }
-        } catch (IllegalAccessException | NoSuchFieldException | ExecutionException |
-                ResourceUnavailableException | ResourceAllocationException | InsufficientCapacityException ex) {
+        } catch (IllegalAccessException | NoSuchFieldException | OperationTimedoutException | ResourceUnavailableException | InsufficientCapacityException ex) {
             throw new ManagementServerException(String.format("Failed to start VM in the Kubernetes cluster : %s", kubernetesCluster.getName()), ex);
         }
 
@@ -326,8 +332,8 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
             ResourceUnavailableException, InsufficientCapacityException {
         List<UserVm> nodes = new ArrayList<>();
         for (int i = offset + 1; i <= nodeCount; i++) {
-            UserVm vm = createKubernetesNode(publicIpAddress, i);
-            addKubernetesClusterVm(kubernetesCluster.getId(), vm.getId());
+            UserVm vm = createKubernetesNode(publicIpAddress);
+            addKubernetesClusterVm(kubernetesCluster.getId(), vm.getId(), false);
             if (kubernetesCluster.getNodeRootDiskSize() > 0) {
                 resizeNodeVolume(vm);
             }
@@ -349,7 +355,7 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         return provisionKubernetesClusterNodeVms(nodeCount, 0, publicIpAddress);
     }
 
-    protected UserVm createKubernetesNode(String joinIp, int nodeInstance) throws ManagementServerException,
+    protected UserVm createKubernetesNode(String joinIp) throws ManagementServerException,
             ResourceUnavailableException, InsufficientCapacityException {
         UserVm nodeVm = null;
         DataCenter zone = dataCenterDao.findById(kubernetesCluster.getZoneId());
@@ -363,7 +369,8 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         if (rootDiskSize > 0) {
             customParameterMap.put("rootdisksize", String.valueOf(rootDiskSize));
         }
-        String hostName = getKubernetesClusterNodeAvailableName(String.format("%s-node-%s", kubernetesClusterNodeNamePrefix, nodeInstance));
+        String suffix = Long.toHexString(System.currentTimeMillis());
+        String hostName = String.format("%s-node-%s", kubernetesClusterNodeNamePrefix, suffix);
         String k8sNodeConfig = null;
         try {
             k8sNodeConfig = getKubernetesNodeConfig(joinIp, Hypervisor.HypervisorType.VMware.equals(clusterTemplate.getHypervisorType()));
@@ -374,7 +381,7 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         nodeVm = userVmService.createAdvancedVirtualMachine(zone, serviceOffering, clusterTemplate, networkIds, owner,
                 hostName, hostName, null, null, null,
                 Hypervisor.HypervisorType.None, BaseCmd.HTTPMethod.POST, base64UserData, kubernetesCluster.getKeyPair(),
-                null, addrs, null, null, null, customParameterMap, null, null, null, null);
+                null, addrs, null, null, null, customParameterMap, null, null, null, null, true, UserVmManager.CKS_NODE);
         if (LOGGER.isInfoEnabled()) {
             LOGGER.info(String.format("Created node VM : %s, %s in the Kubernetes cluster : %s", hostName, nodeVm.getUuid(), kubernetesCluster.getName()));
         }
@@ -453,7 +460,6 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
                 final Ip vmIp = new Ip(vmNic.getIPv4Address());
                 final long vmIdFinal = vmId;
                 final int srcPortFinal = firewallRuleSourcePortStart + i;
-
                 PortForwardingRuleVO pfRule = Transaction.execute(new TransactionCallbackWithException<PortForwardingRuleVO, NetworkRuleConflictException>() {
                     @Override
                     public PortForwardingRuleVO doInTransaction(TransactionStatus status) throws NetworkRuleConflictException {
@@ -519,6 +525,17 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         }
     }
 
+    protected void removePortForwardingRules(final IpAddress publicIp, final Network network, final Account account, int startPort, int endPort)
+        throws ResourceUnavailableException {
+        List<PortForwardingRuleVO> pfRules = portForwardingRulesDao.listByNetwork(network.getId());
+        for (PortForwardingRuleVO pfRule : pfRules) {
+            if (startPort <= pfRule.getSourcePortStart() && pfRule.getSourcePortStart() <= endPort) {
+                portForwardingRulesDao.remove(pfRule.getId());
+            }
+        }
+        rulesService.applyPortForwardingRules(publicIp.getId(), account);
+    }
+
     protected void removeLoadBalancingRule(final IpAddress publicIp, final Network network,
                                            final Account account, final int port) throws ResourceUnavailableException {
         List<LoadBalancerVO> rules = loadBalancerDao.listByIpAddress(publicIp.getId());
@@ -548,13 +565,96 @@ public class KubernetesClusterResourceModifierActionWorker extends KubernetesClu
         return prefix;
     }
 
-    protected String getKubernetesClusterNodeAvailableName(final String hostName) {
-        String name = hostName;
-        int suffix = 1;
-        while (vmInstanceDao.findVMByHostName(name) != null) {
-            name = String.format("%s-%d", hostName, suffix);
-            suffix++;
+    protected KubernetesClusterVO updateKubernetesClusterEntry(final Long cores, final Long memory,
+        final Long size, final Long serviceOfferingId, final Boolean autoscaleEnabled, final Long minSize, final Long maxSize) {
+        return Transaction.execute(new TransactionCallback<KubernetesClusterVO>() {
+                @Override
+                public KubernetesClusterVO doInTransaction(TransactionStatus status) {
+                KubernetesClusterVO updatedCluster = kubernetesClusterDao.createForUpdate(kubernetesCluster.getId());
+                if (cores != null) {
+                    updatedCluster.setCores(cores);
+                }
+                if (memory != null) {
+                    updatedCluster.setMemory(memory);
+                }
+                if (size != null) {
+                    updatedCluster.setNodeCount(size);
+                }
+                if (serviceOfferingId != null) {
+                    updatedCluster.setServiceOfferingId(serviceOfferingId);
+                }
+                if (autoscaleEnabled != null) {
+                    updatedCluster.setAutoscalingEnabled(autoscaleEnabled.booleanValue());
+                }
+                updatedCluster.setMinSize(minSize);
+                updatedCluster.setMaxSize(maxSize);
+                return kubernetesClusterDao.persist(updatedCluster);
+            }
+        });
+    }
+
+    private KubernetesClusterVO updateKubernetesClusterEntry(final Boolean autoscaleEnabled, final Long minSize, final Long maxSize) throws CloudRuntimeException {
+        KubernetesClusterVO kubernetesClusterVO = updateKubernetesClusterEntry(null, null, null, null, autoscaleEnabled, minSize, maxSize);
+        if (kubernetesClusterVO == null) {
+            logTransitStateAndThrow(Level.ERROR, String.format("Scaling Kubernetes cluster %s failed, unable to update Kubernetes cluster",
+                    kubernetesCluster.getName()), kubernetesCluster.getId(), KubernetesCluster.Event.OperationFailed);
+        }
+        return kubernetesClusterVO;
+    }
+
+    protected boolean autoscaleCluster(boolean enable, Long minSize, Long maxSize) {
+        if (!kubernetesCluster.getState().equals(KubernetesCluster.State.Scaling)) {
+            stateTransitTo(kubernetesCluster.getId(), KubernetesCluster.Event.AutoscaleRequested);
+        }
+
+        File pkFile = getManagementServerSshPublicKeyFile();
+        Pair<String, Integer> publicIpSshPort = getKubernetesClusterServerIpSshPort(null);
+        publicIpAddress = publicIpSshPort.first();
+        sshPort = publicIpSshPort.second();
+
+        try {
+            if (enable) {
+                String command = String.format("sudo /opt/bin/autoscale-kube-cluster -i %s -e -M %d -m %d",
+                    kubernetesCluster.getUuid(), maxSize, minSize);
+                Pair<Boolean, String> result = SshHelper.sshExecute(publicIpAddress, sshPort, CLUSTER_NODE_VM_USER,
+                    pkFile, null, command, 10000, 10000, 60000);
+
+                // Maybe the file isn't present. Try and copy it
+                if (!result.first()) {
+                    logMessage(Level.INFO, "Autoscaling files missing. Adding them now", null);
+                    retrieveScriptFiles();
+                    copyScripts(publicIpAddress, sshPort);
+
+                    if (!createCloudStackSecret(keys)) {
+                        logTransitStateAndThrow(Level.ERROR, String.format("Failed to setup keys for Kubernetes cluster %s",
+                            kubernetesCluster.getName()), kubernetesCluster.getId(), KubernetesCluster.Event.OperationFailed);
+                    }
+
+                    // If at first you don't succeed ...
+                    result = SshHelper.sshExecute(publicIpAddress, sshPort, CLUSTER_NODE_VM_USER,
+                        pkFile, null, command, 10000, 10000, 60000);
+                    if (!result.first()) {
+                        throw new CloudRuntimeException(result.second());
+                    }
+                }
+                updateKubernetesClusterEntry(true, minSize, maxSize);
+            } else {
+                Pair<Boolean, String> result = SshHelper.sshExecute(publicIpAddress, sshPort, CLUSTER_NODE_VM_USER,
+                    pkFile, null, String.format("sudo /opt/bin/autoscale-kube-cluster -d"),
+                        10000, 10000, 60000);
+                if (!result.first()) {
+                    throw new CloudRuntimeException(result.second());
+                }
+                updateKubernetesClusterEntry(false, null, null);
+            }
+            return true;
+        } catch (Exception e) {
+            String msg = String.format("Failed to autoscale Kubernetes cluster: %s : %s", kubernetesCluster.getName(), e.getMessage());
+            logAndThrow(Level.ERROR, msg);
+            return false;
+        } finally {
+            // Deploying the autoscaler might fail but it can be deployed manually too, so no need to go to an alert state
+            stateTransitTo(kubernetesCluster.getId(), KubernetesCluster.Event.OperationSucceeded);
         }
-        return name;
     }
 }
