@@ -36,7 +36,7 @@ import org.apache.commons.collections.MapUtils;
 import org.apache.log4j.Logger;
 
 import com.cloud.agent.AgentManager;
-import com.cloud.agent.api.Answer;
+import com.cloud.agent.api.HandleConfigDriveIsoAnswer;
 import com.cloud.agent.api.HandleConfigDriveIsoCommand;
 import com.cloud.agent.api.to.DiskTO;
 import com.cloud.configuration.ConfigurationManager;
@@ -254,10 +254,6 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
 
         final boolean canHandle = canHandle(network.getTrafficType());
 
-        if (canHandle) {
-            storePasswordInVmDetails(vm);
-        }
-
         return canHandle;
     }
 
@@ -338,7 +334,16 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
         if (_networkModel.getUserDataUpdateProvider(network).getProvider().equals(Provider.ConfigDrive)) {
             LOG.trace(String.format("[prepareMigration] for vm: %s", vm.getInstanceName()));
             try {
-                addPasswordAndUserdata(network, nic, vm, dest, context);
+                if (isConfigDriveIsoOnHostCache(vm.getId())) {
+                    vm.setConfigDriveLocation(Location.HOST);
+                    configureConfigDriveData(vm, nic, dest);
+
+                    // Create the config drive on dest host cache
+                    createConfigDriveIsoOnHostCache(vm, dest.getHost().getId());
+                } else {
+                    vm.setConfigDriveLocation(getConfigDriveLocation(vm.getId()));
+                    addPasswordAndUserdata(network, nic, vm, dest, context);
+                }
             } catch (InsufficientCapacityException | ResourceUnavailableException e) {
                 LOG.error("Failed to add config disk drive due to: ", e);
                 return false;
@@ -349,10 +354,28 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
 
     @Override
     public void rollbackMigration(NicProfile nic, Network network, VirtualMachineProfile vm, ReservationContext src, ReservationContext dst) {
+        try {
+            if (isConfigDriveIsoOnHostCache(vm.getId())) {
+                vm.setConfigDriveLocation(Location.HOST);
+                // Delete the config drive on dest host cache
+                deleteConfigDriveIsoOnHostCache(vm.getVirtualMachine(), vm.getHostId());
+            }
+        } catch (ConcurrentOperationException | ResourceUnavailableException e) {
+            LOG.error("rollbackMigration failed.", e);
+        }
     }
 
     @Override
     public void commitMigration(NicProfile nic, Network network, VirtualMachineProfile vm, ReservationContext src, ReservationContext dst) {
+        try {
+            if (isConfigDriveIsoOnHostCache(vm.getId())) {
+                vm.setConfigDriveLocation(Location.HOST);
+                // Delete the config drive on src host cache
+                deleteConfigDriveIsoOnHostCache(vm.getVirtualMachine(), vm.getHostId());
+            }
+        } catch (ConcurrentOperationException | ResourceUnavailableException e) {
+            LOG.error("commitMigration failed.", e);
+        }
     }
 
     private void recreateConfigDriveIso(NicProfile nic, Network network, VirtualMachineProfile vm, DeployDestination dest) throws ResourceUnavailableException {
@@ -383,7 +406,8 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
 
     private DataStore findDataStore(VirtualMachineProfile profile, DeployDestination dest) {
         DataStore dataStore = null;
-        if (VirtualMachineManager.VmConfigDriveOnPrimaryPool.value()) {
+        if (VirtualMachineManager.VmConfigDriveOnPrimaryPool.valueIn(dest.getDataCenter().getId()) ||
+                VirtualMachineManager.VmConfigDriveForceHostCacheUse.valueIn(dest.getDataCenter().getId())) {
             if(MapUtils.isNotEmpty(dest.getStorageForDisks())) {
                 dataStore = getPlannedDataStore(dest, dataStore);
             }
@@ -472,12 +496,86 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
         } else {
             agentId = dest.getHost().getId();
         }
-        if (!VirtualMachineManager.VmConfigDriveOnPrimaryPool.value()) {
+        if (!VirtualMachineManager.VmConfigDriveOnPrimaryPool.valueIn(dest.getDataCenter().getId()) &&
+                !VirtualMachineManager.VmConfigDriveForceHostCacheUse.valueIn(dest.getDataCenter().getId())) {
             agentId = findAgentIdForImageStore(dataStore);
         }
         return agentId;
     }
 
+    private Location getConfigDriveLocation(long vmId) {
+        final UserVmDetailVO vmDetailConfigDriveLocation = _userVmDetailsDao.findDetail(vmId, VmDetailConstants.CONFIG_DRIVE_LOCATION);
+        if (vmDetailConfigDriveLocation != null) {
+            if (Location.HOST.toString().equalsIgnoreCase(vmDetailConfigDriveLocation.getValue())) {
+                return Location.HOST;
+            } else if (Location.PRIMARY.toString().equalsIgnoreCase(vmDetailConfigDriveLocation.getValue())) {
+                return Location.PRIMARY;
+            } else {
+                return Location.SECONDARY;
+            }
+        }
+        return Location.SECONDARY;
+    }
+
+    private boolean isConfigDriveIsoOnHostCache(long vmId) {
+        final UserVmDetailVO vmDetailConfigDriveLocation = _userVmDetailsDao.findDetail(vmId, VmDetailConstants.CONFIG_DRIVE_LOCATION);
+        if (vmDetailConfigDriveLocation != null && Location.HOST.toString().equalsIgnoreCase(vmDetailConfigDriveLocation.getValue())) {
+            return true;
+        }
+        return false;
+    }
+
+    private boolean createConfigDriveIsoOnHostCache(VirtualMachineProfile profile, Long hostId) throws ResourceUnavailableException {
+        if (hostId == null) {
+            throw new ResourceUnavailableException("Config drive iso creation failed, dest host not available",
+                    ConfigDriveNetworkElement.class, 0L);
+        }
+
+        LOG.debug("Creating config drive ISO for vm: " + profile.getInstanceName() + " on host: " + hostId);
+
+        final String isoFileName = ConfigDrive.configIsoFileName(profile.getInstanceName());
+        final String isoPath = ConfigDrive.createConfigDrivePath(profile.getInstanceName());
+        final String isoData = ConfigDriveBuilder.buildConfigDrive(profile.getVmData(), isoFileName, profile.getConfigDriveLabel());
+        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, isoData, null, false, true, true);
+
+        final HandleConfigDriveIsoAnswer answer = (HandleConfigDriveIsoAnswer) agentManager.easySend(hostId, configDriveIsoCommand);
+        if (answer == null) {
+            throw new CloudRuntimeException("Unable to get an answer to handle config drive creation for vm: " + profile.getInstanceName() + " on host: " + hostId);
+        }
+
+        if (!answer.getResult()) {
+            throw new ResourceUnavailableException(String.format("Config drive iso creation failed, details: %s",
+                    answer.getDetails()), ConfigDriveNetworkElement.class, 0L);
+        }
+
+        profile.setConfigDriveLocation(answer.getConfigDriveLocation());
+        _userVmDetailsDao.addDetail(profile.getId(), VmDetailConstants.CONFIG_DRIVE_LOCATION, answer.getConfigDriveLocation().toString(), false);
+        addConfigDriveDisk(profile, null);
+        return true;
+    }
+
+    private boolean deleteConfigDriveIsoOnHostCache(final VirtualMachine vm, final Long hostId) throws ResourceUnavailableException {
+        if (hostId == null) {
+            throw new ResourceUnavailableException("Config drive iso deletion failed, host not available",
+                    ConfigDriveNetworkElement.class, 0L);
+        }
+
+        LOG.debug("Deleting config drive ISO for vm: " + vm.getInstanceName() + " on host: " + hostId);
+        final String isoPath = ConfigDrive.createConfigDrivePath(vm.getInstanceName());
+        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, null, null, false, true, false);
+
+        final HandleConfigDriveIsoAnswer answer = (HandleConfigDriveIsoAnswer) agentManager.easySend(hostId, configDriveIsoCommand);
+        if (answer == null) {
+            throw new CloudRuntimeException("Unable to get an answer to handle config drive deletion for vm: " + vm.getInstanceName() + " on host: " + hostId);
+        }
+
+        if (!answer.getResult()) {
+            LOG.error("Failed to remove config drive for instance: " + vm.getInstanceName());
+            return false;
+        }
+        return true;
+    }
+
     private boolean createConfigDriveIso(VirtualMachineProfile profile, DeployDestination dest, DiskTO disk) throws ResourceUnavailableException {
         DataStore dataStore = getDatastoreForConfigDriveIso(disk, profile, dest);
 
@@ -492,13 +590,17 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
         final String isoFileName = ConfigDrive.configIsoFileName(profile.getInstanceName());
         final String isoPath = ConfigDrive.createConfigDrivePath(profile.getInstanceName());
         final String isoData = ConfigDriveBuilder.buildConfigDrive(profile.getVmData(), isoFileName, profile.getConfigDriveLabel());
-        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, isoData, dataStore.getTO(), true);
+        boolean useHostCacheOnUnsupportedPool = VirtualMachineManager.VmConfigDriveUseHostCacheOnUnsupportedPool.valueIn(dest.getDataCenter().getId());
+        boolean preferHostCache = VirtualMachineManager.VmConfigDriveForceHostCacheUse.valueIn(dest.getDataCenter().getId());
+        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, isoData, dataStore.getTO(), useHostCacheOnUnsupportedPool, preferHostCache, true);
 
-        final Answer answer = agentManager.easySend(agentId, configDriveIsoCommand);
+        final HandleConfigDriveIsoAnswer answer = (HandleConfigDriveIsoAnswer) agentManager.easySend(agentId, configDriveIsoCommand);
         if (!answer.getResult()) {
             throw new ResourceUnavailableException(String.format("Config drive iso creation failed, details: %s",
                     answer.getDetails()), ConfigDriveNetworkElement.class, 0L);
         }
+        profile.setConfigDriveLocation(answer.getConfigDriveLocation());
+        _userVmDetailsDao.addDetail(profile.getId(), VmDetailConstants.CONFIG_DRIVE_LOCATION, answer.getConfigDriveLocation().toString(), false);
         addConfigDriveDisk(profile, dataStore);
         return true;
     }
@@ -526,28 +628,37 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
     }
 
     private boolean deleteConfigDriveIso(final VirtualMachine vm) throws ResourceUnavailableException {
-        DataStore dataStore = _dataStoreMgr.getImageStoreWithFreeCapacity(vm.getDataCenterId());
-        Long agentId = findAgentIdForImageStore(dataStore);
+        Long hostId  = (vm.getHostId() != null) ? vm.getHostId() : vm.getLastHostId();
+        Location location = getConfigDriveLocation(vm.getId());
+        if (location == Location.HOST) {
+            return deleteConfigDriveIsoOnHostCache(vm, hostId);
+        }
 
-        if (VirtualMachineManager.VmConfigDriveOnPrimaryPool.value()) {
+        Long agentId = null;
+        DataStore dataStore = null;
+
+        if (location == Location.SECONDARY) {
+            dataStore = _dataStoreMgr.getImageStoreWithFreeCapacity(vm.getDataCenterId());
+            agentId = findAgentIdForImageStore(dataStore);
+        } else if (location == Location.PRIMARY) {
             List<VolumeVO> volumes = _volumeDao.findByInstanceAndType(vm.getId(), Volume.Type.ROOT);
             if (volumes != null && volumes.size() > 0) {
                 dataStore = _dataStoreMgr.getDataStore(volumes.get(0).getPoolId(), DataStoreRole.Primary);
             }
-            agentId = (vm.getHostId() != null) ? vm.getHostId() : vm.getLastHostId();
+            agentId = hostId;
         }
 
         if (agentId == null || dataStore == null) {
-            throw new ResourceUnavailableException("Config drive iso creation failed, agent or datastore not available",
+            throw new ResourceUnavailableException("Config drive iso deletion failed, agent or datastore not available",
                     ConfigDriveNetworkElement.class, 0L);
         }
 
         LOG.debug("Deleting config drive ISO for vm: " + vm.getInstanceName());
 
         final String isoPath = ConfigDrive.createConfigDrivePath(vm.getInstanceName());
-        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, null, dataStore.getTO(), false);
+        final HandleConfigDriveIsoCommand configDriveIsoCommand = new HandleConfigDriveIsoCommand(isoPath, null, dataStore.getTO(), false, false, false);
 
-        final Answer answer = agentManager.easySend(agentId, configDriveIsoCommand);
+        final HandleConfigDriveIsoAnswer answer = (HandleConfigDriveIsoAnswer) agentManager.easySend(agentId, configDriveIsoCommand);
         if (!answer.getResult()) {
             LOG.error("Failed to remove config drive for instance: " + vm.getInstanceName());
             return false;
@@ -566,11 +677,13 @@ public class ConfigDriveNetworkElement extends AdapterBase implements NetworkEle
         }
         if (!isoAvailable) {
             TemplateObjectTO dataTO = new TemplateObjectTO();
-            if (dataStore == null) {
+            if (dataStore == null && !isConfigDriveIsoOnHostCache(profile.getId())) {
                 throw new ResourceUnavailableException("Config drive disk add failed, datastore not available",
                         ConfigDriveNetworkElement.class, 0L);
+            } else if (dataStore != null) {
+                dataTO.setDataStore(dataStore.getTO());
             }
-            dataTO.setDataStore(dataStore.getTO());
+
             dataTO.setUuid(profile.getUuid());
             dataTO.setPath(isoPath);
             dataTO.setFormat(Storage.ImageFormat.ISO);
