@@ -20,6 +20,8 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import com.cloud.offering.ServiceOffering;
+import com.cloud.template.VirtualMachineTemplate;
 import org.apache.cloudstack.api.BaseCmd.HTTPMethod;
 import org.apache.cloudstack.framework.config.ConfigKey;
 
@@ -55,6 +57,8 @@ public interface UserVmManager extends UserVmService {
 
     static final int MAX_USER_DATA_LENGTH_BYTES = 2048;
 
+    public  static  final String CKS_NODE = "cksnode";
+
     /**
      * @param hostId get all of the virtual machines that belong to one host.
      * @return collection of VirtualMachine.
@@ -96,6 +100,8 @@ public interface UserVmManager extends UserVmService {
 
     void removeInstanceFromInstanceGroup(long vmId);
 
+    boolean isVMUsingLocalStorage(VMInstanceVO vm);
+
     boolean expunge(UserVmVO vm, long callerUserId, Account caller);
 
     Pair<UserVmVO, Map<VirtualMachineProfile.Param, Object>> startVirtualMachine(long vmId, Long hostId, Map<VirtualMachineProfile.Param, Object> additionalParams, String deploymentPlannerToUse)
@@ -122,4 +128,7 @@ public interface UserVmManager extends UserVmService {
     void persistDeviceBusInfo(UserVmVO paramUserVmVO, String paramString);
 
     HashMap<Long, List<VmNetworkStatsEntry>> getVmNetworkStatistics(long hostId, String hostName, List<Long> vmIds);
+
+    boolean checkIfDynamicScalingCanBeEnabled(VirtualMachine vm, ServiceOffering offering, VirtualMachineTemplate template, Long zoneId);
+
 }
