@@ -19,6 +19,7 @@ package com.cloud.vm;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
+import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyLong;
 import static org.mockito.ArgumentMatchers.anyMap;
 import static org.mockito.ArgumentMatchers.anyString;
@@ -32,12 +33,15 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import com.cloud.configuration.Resource;
 import com.cloud.hypervisor.Hypervisor;
 import com.cloud.storage.DiskOfferingVO;
 import com.cloud.storage.VMTemplateVO;
 import com.cloud.storage.VolumeVO;
 import com.cloud.storage.dao.DiskOfferingDao;
 import com.cloud.storage.dao.VMTemplateDao;
+import com.cloud.user.ResourceLimitService;
+import com.cloud.user.dao.AccountDao;
 import org.apache.cloudstack.api.BaseCmd.HTTPMethod;
 import org.apache.cloudstack.api.command.user.vm.UpdateVMCmd;
 import org.apache.cloudstack.api.command.user.volume.ResizeVolumeCmd;
@@ -145,6 +149,12 @@ public class UserVmManagerImplTest {
     @Mock
     private VMTemplateDao templateDao;
 
+    @Mock
+    private AccountDao accountDao;
+
+    @Mock
+    ResourceLimitService resourceLimitMgr;
+
     private long vmId = 1l;
 
     private static final long GiB_TO_BYTES = 1024 * 1024 * 1024;
@@ -167,6 +177,8 @@ public class UserVmManagerImplTest {
         CallContext.register(callerUser, callerAccount);
 
         customParameters.put(VmDetailConstants.ROOT_DISK_SIZE, "123");
+        lenient().doNothing().when(resourceLimitMgr).incrementResourceCount(anyLong(), any(Resource.ResourceType.class));
+        lenient().doNothing().when(resourceLimitMgr).decrementResourceCount(anyLong(), any(Resource.ResourceType.class), anyLong());
     }
 
     @After
@@ -280,6 +292,7 @@ public class UserVmManagerImplTest {
 
     @Test
     public void updateVirtualMachineTestCleanUpFalseAndDetailsEmpty() throws ResourceUnavailableException, InsufficientCapacityException {
+        Mockito.when(accountDao.findById(Mockito.anyLong())).thenReturn(callerAccount);
         prepareAndExecuteMethodDealingWithDetails(false, false);
     }
 
@@ -288,6 +301,10 @@ public class UserVmManagerImplTest {
 
         ServiceOffering offering = getSvcoffering(512);
         Mockito.when(_serviceOfferingDao.findById(Mockito.anyLong(), Mockito.anyLong())).thenReturn((ServiceOfferingVO) offering);
+        Mockito.when(_serviceOfferingDao.findByIdIncludingRemoved(Mockito.anyLong(), Mockito.anyLong())).thenReturn((ServiceOfferingVO) offering);
+        ServiceOfferingVO currentServiceOffering = Mockito.mock(ServiceOfferingVO.class);
+        Mockito.lenient().when(currentServiceOffering.getCpu()).thenReturn(1);
+        Mockito.lenient().when(currentServiceOffering.getRamSize()).thenReturn(512);
 
         List<NicVO> nics = new ArrayList<>();
         NicVO nic1 = mock(NicVO.class);
@@ -303,7 +320,6 @@ public class UserVmManagerImplTest {
         }
         Mockito.when(updateVmCommand.getDetails()).thenReturn(details);
         Mockito.when(updateVmCommand.isCleanupDetails()).thenReturn(cleanUpDetails);
-
         configureDoNothingForDetailsMethod();
 
         userVmManagerImpl.updateVirtualMachine(updateVmCommand);
