@@ -19,7 +19,7 @@
 """
 #Import Local Modules
 from nose.plugins.attrib import attr
-from marvin.cloudstackTestCase import cloudstackTestCase, unittest
+from marvin.cloudstackTestCase import cloudstackTestCase
 from marvin.lib.base import (stopRouter,
                                         startRouter,
                                         Account,
@@ -38,7 +38,6 @@ from marvin.lib.common import (get_domain,
                                         get_zone,
                                         get_template,
                                         list_routers)
-from marvin.lib.utils import cleanup_resources
 import socket
 import time
 
@@ -205,14 +204,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
 
     @classmethod
     def tearDownClass(cls):
-        try:
-            #Cleanup resources used
-            cleanup_resources(cls.api_client, cls._cleanup)
-        except Exception as e:
-            print ("Warning: Exception during cleanup : %s" % e)
-            #raise Exception("Warning: Exception during cleanup : %s" % e)
-        return
-
+        super(TestVPCNetworkLBRules, cls).tearDownClass()
 
     def setUp(self):
         self.apiclient = self.testClient.getApiClient()
@@ -228,7 +220,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                     self.apiclient,
                                     self.services["vpc_offering"]
                                     )
-
         self.cleanup.append(self.vpc_off)
         self.debug("Enabling the VPC offering created")
         self.vpc_off.update(self.apiclient, state='Enabled')
@@ -243,16 +234,11 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                         account=self.account.name,
                         domainid=self.account.domainid
                         )
+        self.cleanup.append(self.vpc)
         return
 
     def tearDown(self):
-        try:
-            #Clean up, terminate the created network offerings
-            cleanup_resources(self.apiclient, self.cleanup)
-        except Exception as e:
-            self.debug("Warning: Exception during cleanup : %s" % e)
-            #raise Exception("Warning: Exception during cleanup : %s" % e)
-        return
+        super(TestVPCNetworkLBRules, self).tearDown()
 
     def get_Router_For_VPC(self):
         routers = list_routers(self.apiclient,
@@ -270,7 +256,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         router = routers[0]
         return router
 
-
     def stop_VPC_VRouter(self):
         router = self.get_Router_For_VPC()
         self.debug("Stopping router ID: %s" % router.id)
@@ -315,35 +300,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                     "Check list router response for router state"
                     )
 
-    def check_ssh_into_vm(self, vm, public_ip, testnegative=False):
-        self.debug("Checking if we can SSH into VM=%s on public_ip=%s" % (vm.name, public_ip.ipaddress.ipaddress))
-        try:
-            vm.get_ssh_client(ipaddress=public_ip.ipaddress.ipaddress)
-            if not testnegative:
-                self.debug("SSH into VM=%s on public_ip=%s is successfully" % (vm.name, public_ip.ipaddress.ipaddress))
-            else:
-                self.fail("SSH into VM=%s on public_ip=%s is successfully" % (vm.name, public_ip.ipaddress.ipaddress))
-        except:
-            if not testnegative:
-                self.fail("Failed to SSH into VM - %s" % (public_ip.ipaddress.ipaddress))
-            else:
-                self.debug("Failed to SSH into VM - %s" % (public_ip.ipaddress.ipaddress))
-
-    def check_wget_from_vm(self, vm, public_ip, testnegative=False):
-        import urllib
-        self.debug("Checking if we can wget from a VM=%s http server on public_ip=%s"  % (vm.name, public_ip.ipaddress.ipaddress))
-        try:
-            urllib.urlretrieve("http://%s/test.html" % public_ip.ipaddress.ipaddress, filename="test.html")
-            if not testnegative:
-                self.debug("Successful to wget from VM=%s http server on public_ip=%s" % (vm.name, public_ip.ipaddress.ipaddress))
-            else:
-                self.fail("Successful to wget from VM=%s http server on public_ip=%s" % (vm.name, public_ip.ipaddress.ipaddress))
-        except Exception, e:
-            if not testnegative:
-                self.fail("Failed to wget from VM=%s http server on public_ip=%s because of %s" % (vm.name, public_ip.ipaddress.ipaddress, e))
-            else:
-                self.debug("Failed to wget from VM=%s http server on public_ip=%s because of %s" % (vm.name, public_ip.ipaddress.ipaddress, e))
-
     def create_StaticNatRule_For_VM(self, vm, public_ip, network):
         self.debug("Enabling static NAT for IP: %s" %
                                             public_ip.ipaddress.ipaddress)
@@ -360,26 +316,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
             self.fail("Failed to enable static NAT on IP: %s - %s" % (
                                         public_ip.ipaddress.ipaddress, e))
 
-    def create_NatRule_For_VM(self, vm, public_ip, network):
-        self.debug("Creatinng NAT rule in network for vm with public IP")
-        nat_rule = NATRule.create(self.apiclient,
-                                vm,
-                                self.services["natrule"],
-                                ipaddressid=public_ip.ipaddress.id,
-                                openfirewall=False,
-                                networkid=network.id,
-                                vpcid=self.vpc.id
-                                )
-
-        self.debug("Adding NetwrokACl rules to make NAT rule accessible")
-        nwacl_nat = NetworkACL.create(self.apiclient,
-                                    networkid=network.id,
-                                    services=self.services["natrule"],
-                                    traffictype='Ingress'
-                                    )
-        self.debug('nwacl_nat=%s' % nwacl_nat.__dict__)
-        return nat_rule
-
     def acquire_Public_IP(self, network):
         self.debug("Associating public IP for network: %s" % network.name)
         public_ip = PublicIPAddress.create(self.apiclient,
@@ -389,6 +325,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                 networkid=None, #network.id,
                                 vpcid=self.vpc.id
                                 )
+        self.cleanup.append(public_ip)
         self.debug("Associated %s with network %s" % (public_ip.ipaddress.ipaddress,
                                         network.id
                                         ))
@@ -401,7 +338,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                     self.apiclient,
                                     self.services["vpc_offering"]
                                     )
-
         self.cleanup.append(vpc_off)
         self.debug("Enabling the VPC offering created")
         vpc_off.update(self.apiclient, state='Enabled')
@@ -416,9 +352,10 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                         account=self.account.name,
                         domainid=self.account.domainid
                         )
+        self.cleanup.append(vpc)
         return vpc
 
-    def create_Network(self, net_offerring, gateway='10.1.1.1',vpc=None):
+    def create_network(self, net_offerring, gateway='10.1.1.1',vpc=None):
         try:
             self.debug('Create NetworkOffering')
             net_offerring["name"] = "NET_OFF-" + str(gateway)
@@ -426,9 +363,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                             net_offerring,
                                             conservemode=False
                                         )
+            self.cleanup.append(nw_off)
             # Enable Network offering
             nw_off.update(self.apiclient, state='Enabled')
-            self.cleanup.append(nw_off)
             self.debug('Created and Enabled NetworkOffering')
 
             self.services["network"]["name"] = "NETWORK-" + str(gateway)
@@ -442,6 +379,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                     gateway=gateway,
                                     vpcid=vpc.id if vpc else self.vpc.id
                                     )
+            self.cleanup.append(obj_network)
             self.debug("Created network with ID: %s" % obj_network.id)
             return obj_network
         except:
@@ -459,6 +397,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                     networkids=[str(network.id)],
                                     hostid=host_id
                                     )
+            self.cleanup.append(vm)
             self.debug('Created VM=%s in network=%s' % (vm.id, network.name))
 
             return vm
@@ -492,24 +431,13 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                     networkid=network.id,
                                     traffictype='Ingress'
                                     )
+        self.cleanup.append(nwacl_nat)
         self.debug('nwacl_nat=%s' % nwacl_nat.__dict__)
         return lb_rule
 
-    def create_egress_Internet_Rule(self, network):
-        self.debug("Adding Egress rules to network %s and %s to allow access to internet" % (network.name,self.services["http_rule"]))
-        nwacl_internet_1 = NetworkACL.create(
-                                self.apiclient,
-                                networkid=network.id,
-                                services=self.services["http_rule"],
-                                traffictype='Egress'
-                                )
-
-        return nwacl_internet_1
-
     @attr(tags=["advanced", "intervlan"], required_hardware="false")
     def test_01_VPC_LBRulesListing(self):
         """ Test case no 210 and 227: List Load Balancing Rules belonging to a VPC
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -522,16 +450,17 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 8. Use the Create LB rule for vm3 amd vm4 in network2, should fail
         # because it's no_lb offering
         # 9. List LB rule
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1')
+        network_1 = self.create_network(self.services["network_offering"])
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1')
         self.debug("deploying VMs in network: %s" % network_2.name)
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_2)
         vm_4 = self.create_VM_in_Network(network_2)
         public_ip_1 = self.acquire_Public_IP(network_1)
-        lb_rule1 = self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2]) #
+        lb_rule1 = self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         public_ip_2 = self.acquire_Public_IP(network_2)
         with self.assertRaises(Exception):
             self.create_LB_Rule(public_ip_2, network_2, [vm_3, vm_4])
@@ -540,7 +469,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                         id=lb_rule1.id,
                                         listall=True
                                         )
-        self.failIfEqual(lb_rules,
+        self.assertNotEqual(lb_rules,
                         None,
                         "Failed to list the LB Rule"
                         )
@@ -548,7 +477,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
                                 networkid=network_1.id,
                                 listall=True
                                 )
-        self.failIfEqual(vms,
+        self.assertNotEqual(vms,
                         None,
                         "Failed to list the VMs in network=%s" % network_1.name
                         )
@@ -558,7 +487,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_02_VPC_CreateLBRuleInMultipleNetworks(self):
         """ Test Create LB rules for 1 network which is part of a two/multiple virtual networks of a
             VPC using a new Public IP Address available with the VPC when the Virtual Router is in Running State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -568,14 +496,16 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 5. Use the Create LB rule for vm1 and vm2 in network1.
         # 6. Add vm3 to LB rule.
         # 7. wget a file and check for LB rule.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
         public_ip_1 = self.acquire_Public_IP(network_1)
         lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2], self.services["lbrule_http"])
         lb_rule.assign(self.apiclient, [vm_3])
+        self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=False)
         return
 
@@ -583,7 +513,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_03_VPC_CreateLBRuleInMultipleNetworksVRStoppedState(self):
         """ Test case no 222 : Create LB rules for a two/multiple virtual networks of a 
             VPC using a new Public IP Address available with the VPC when the Virtual Router is in Stopped State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -594,9 +523,10 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. Use the Create LB rule for vm1 and vm2 in network1.
         # 8. Add vm3 to LB rule.
         # 9. wget a file and check for LB rule.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1')
+        network_1 = self.create_network(self.services["network_offering"])
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1')
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
@@ -612,6 +542,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
 
         self.start_VPC_VRouter(router)
 
+        self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=False)
         return    
 
@@ -619,7 +550,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_04_VPC_CreateLBRuleInMultipleNetworksVRStoppedState(self):
         """ Test case no 222 : Create LB rules for a two/multiple virtual networks of a
             VPC using a new Public IP Address available with the VPC when the Virtual Router is in Stopped State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -630,9 +560,10 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. Use the Create LB rule for vm1 and vm2 in network1.
         # 8. Add vm3 to LB rule.
         # 9. wget a file and check for LB rule.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1')
+        network_1 = self.create_network(self.services["network_offering"])
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1')
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_2)
@@ -642,6 +573,7 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # http://cloudstack.apache.org/docs/en-US/Apache_CloudStack/4.0.2/html/Installation_Guide/configure-vpc.html
         with self.assertRaises(Exception):
             lb_rule.assign(self.apiclient, [vm_3])
+        self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=False)
         return
 
@@ -649,7 +581,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_05_VPC_CreateAndDeleteLBRule(self):
         """ Test case no 214 : Delete few(not all) LB rules for a single virtual network of a
             VPC belonging to a single Public IP Address when the Virtual Router is in Running State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -661,8 +592,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. wget and ssh and check for LB rule.
         # 8. Delete ssh LB Rule.
         # 9. ssh LB should fail.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
@@ -680,7 +612,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_06_VPC_CreateAndDeleteLBRuleVRStopppedState(self):
         """ Test Delete few(not all) LB rules for a single virtual network of
             a VPC belonging to a single Public IP Address when the Virtual Router is in Stopped State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -692,8 +623,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. wget and ssh and check for LB rule.
         # 8. Delete ssh LB Rule.
         # 9. ssh LB should fail.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
@@ -717,7 +649,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_07_VPC_CreateAndDeleteAllLBRule(self):
         """ Test Delete all LB rules for a single virtual network of a
             VPC belonging to a single Public IP Address when the Virtual Router is in Running State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -729,8 +660,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. wget and ssh and check for LB rule.
         # 8. Delete all LB Rule.
         # 9. ssh and http LB should fail.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
@@ -750,7 +682,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_08_VPC_CreateAndDeleteAllLBRuleVRStoppedState(self):
         """ Test Delete all LB rules for a single virtual network of a
             VPC belonging to a single Public IP Address when the Virtual Router is in Stopped State
-        """
 
         # Validate the following
         # 1. Create a VPC with cidr - 10.1.1.1/16
@@ -762,8 +693,12 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 7. wget and ssh and check for LB rule.
         # 8. Delete all LB Rule.
         # 9. ssh and http LB should fail.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        # same as test_07 so
+        self.skipTest("duplicate test")
+
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vm_3 = self.create_VM_in_Network(network_1)
@@ -778,11 +713,10 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         self.check_ssh_into_vm(vm_1, public_ip_1, testnegative=True)
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=True)
         return
-    
+
     @attr(tags=["advanced", "intervlan"], required_hardware="true")
     def test_09_VPC_LBRuleCreateFailMultipleVPC(self):
         """ Test User should not be allowed to create a LB rule for a VM that belongs to a different VPC.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -797,18 +731,20 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 9. wget and check LB Rule
         # 10. create LB rule for vm3 and vm4 in VPC1
         # 11. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
 
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         vpc2 = self.create_VPC()
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1',vpc2)
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1',vpc2)
         vm_3 = self.create_VM_in_Network(network_2)
         vm_4 = self.create_VM_in_Network(network_2)
         public_ip_1 = self.acquire_Public_IP(network_1)
         lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2], self.services["lbrule_http"])
         self.debug('lb_rule=%s' % lb_rule.__dict__)
+        self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=False)
         try:
             lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_3, vm_4], self.services["lbrule_http"])
@@ -820,7 +756,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     @attr(tags=["advanced", "intervlan"], required_hardware="false")
     def test_10_VPC_FailedToCreateLBRuleNonVPCNetwork(self):
         """ Test User should not be allowed to create a LB rule for a VM that does not belong to any VPC.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -834,13 +769,14 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 9. wget and check LB Rule
         # 10. create LB rule for vm3 and vm4 in VPC1
         # 11. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1')
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1')
         vm_3 = self.create_VM_in_Network(network_2)
-        network_3 = self.create_Network(self.services["network_offering_no_lb"], '10.1.3.1')
+        network_3 = self.create_network(self.services["network_offering_no_lb"], '10.1.3.1')
         vm_4 = self.create_VM_in_Network(network_3)
         self.debug('vm_4=%s' % vm_4.id)
         public_ip_1 = self.acquire_Public_IP(network_1)
@@ -858,7 +794,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     def test_11_VPC_LBRuleCreateNotAllowed(self):
         """ Test case no 217 and 236: User should not be allowed to create a LB rule for a
             VM that does not belong to the same network but belongs to the same VPC.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -872,18 +807,20 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 9. wget and check LB Rule
         # 10. create LB rule for vm3 and vm1 in VPC1
         # 11. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
 
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
-        network_2 = self.create_Network(self.services["network_offering_no_lb"], '10.1.2.1')
+        network_2 = self.create_network(self.services["network_offering_no_lb"], '10.1.2.1')
         vm_3 = self.create_VM_in_Network(network_2)
         vm_4 = self.create_VM_in_Network(network_2)
         self.debug('vm_4=%s' % vm_4.id)
         public_ip_1 = self.acquire_Public_IP(network_1)
         lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2], self.services["lbrule_http"])
         self.debug('lb_rule=%s' % lb_rule.__dict__)
+        self.create_LB_Rule(public_ip_1, network_1, [vm_1, vm_2])
         self.check_wget_from_vm(vm_1, public_ip_1, testnegative=False)
         try:
             lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_3, vm_1], self.services["lbrule_http"])
@@ -895,7 +832,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     @attr(tags=["advanced", "intervlan"], required_hardware="false")
     def test_12_VPC_LBRuleCreateFailForRouterIP(self):
         """ Test User should not be allowed to create a LB rule on an Ipaddress that Source Nat enabled.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -906,8 +842,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 6. Get source NAT public ip of router
         # 7. Use the Create LB rule for vm1 and vm2 in network1.
         # 8. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         router = self.get_Router_For_VPC()
@@ -923,7 +860,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     @attr(tags=["advanced", "intervlan"], required_hardware="true")
     def test_13_VPC_LBRuleCreateFailForPFSourceNATIP(self):
         """ Test User should not be allowed to create a LB rule on an Ipaddress that already has a PF rule.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -935,12 +871,13 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 6. Create a PP rule for vm1
         # 7. Use the Create LB rule for vm1 and vm2 in network1.
         # 8. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         public_ip_1 = self.acquire_Public_IP(network_1)
-        nat_rule1  = self.create_NatRule_For_VM(vm_1, public_ip_1, network_1)
+        nat_rule1  = self.create_natrule_for_services(vm_1, public_ip_1, network_1)
         self.check_ssh_into_vm(vm_1, public_ip_1, testnegative=False)
         self.debug('nat_rule1=%s' % nat_rule1.__dict__)
         try:
@@ -953,7 +890,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     @attr(tags=["advanced", "intervlan"], required_hardware="false")
     def test_14_VPC_LBRuleCreateFailForStaticNatRule(self):
         """ Test User should not be allowed to create a LB rule on an Ipaddress that already has a Static Nat rule.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -966,8 +902,9 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 8. Succesessfully wget a file from vm1.
         # 9. Use the Create LB rule for vm1 and vm2 in network1.
         # 10. LB rule creation should fail.
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         public_ip_1 = self.acquire_Public_IP(network_1)
@@ -982,7 +919,6 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
     @attr(tags=["advanced", "intervlan"], required_hardware="false")
     def test_15_VPC_ReleaseIPForLBRuleCreated(self):
         """ Test release Ip address that has a LB rule assigned to it.
-        """
 
         # Validate the following
         # 1. Create a VPC1 with cidr - 10.1.1.1/16
@@ -994,13 +930,15 @@ class TestVPCNetworkLBRules(cloudstackTestCase):
         # 6. Create a StaticNat Rule rule for vm1
         # 7. Use the Create LB rule for vm1 and vm2 in network1.
         # 8. LB rule creation should fail
+        """
 
-        network_1 = self.create_Network(self.services["network_offering"])
+        network_1 = self.create_network(self.services["network_offering"])
         vm_1 = self.create_VM_in_Network(network_1)
         vm_2 = self.create_VM_in_Network(network_1)
         public_ip_1 = self.acquire_Public_IP(network_1)
         lb_rule = self.create_LB_Rule(public_ip_1, network_1, [vm_2, vm_1])
         public_ip_1.delete(self.apiclient)
+        self.cleanup.remove(public_ip_1)
 
         with self.assertRaises(Exception):
             lb_rules = LoadBalancerRule.list(self.apiclient,
