@@ -220,6 +220,7 @@ class TestAccounts(cloudstackTestCase):
             account=account.name,
             domainid=account.domainid
         )
+        self.cleanup.append(user)
         self.debug("Created user: %s" % user.id)
         list_users_response = list_users(
             self.apiclient,
@@ -324,20 +325,20 @@ class TestRemoveUserFromAccount(cloudstackTestCase):
         )
         cls.services["virtual_machine"]["zoneid"] = cls.zone.id
         cls.services["virtual_machine"]["template"] = cls.template.id
+        cls._cleanup = []
 
         cls.service_offering = ServiceOffering.create(
             cls.api_client,
             cls.services["service_offering"]
         )
+        cls._cleanup.append(cls.service_offering)
         # Create an account
         cls.account = Account.create(
             cls.api_client,
             cls.services["account"]
         )
+        cls._cleanup.append(cls.account)
 
-        cls._cleanup = [cls.account,
-                        cls.service_offering,
-                        ]
         return
 
     @classmethod
@@ -380,6 +381,7 @@ class TestRemoveUserFromAccount(cloudstackTestCase):
             domainid=self.account.domainid
         )
         self.debug("Created user: %s" % user_1.id)
+        self.cleanup.append(user_1)
 
         user_2 = User.create(
             self.apiclient,
@@ -419,6 +421,7 @@ class TestRemoveUserFromAccount(cloudstackTestCase):
         # Remove one of the user
         self.debug("Deleting user: %s" % user_1.id)
         user_1.delete(self.apiclient)
+        self.cleanup.remove(user_1)
 
         # Account should exist after deleting user
         accounts_response = list_accounts(
@@ -991,6 +994,7 @@ class TestAddVmToSubDomain(cloudstackTestCase):
             domainid=cls.account_1.domainid,
             serviceofferingid=cls.service_offering.id
         )
+        cls._cleanup.append(cls.vm_1)
 
         cls.vm_2 = VirtualMachine.create(
             cls.api_client,
@@ -1000,6 +1004,7 @@ class TestAddVmToSubDomain(cloudstackTestCase):
             domainid=cls.account_2.domainid,
             serviceofferingid=cls.service_offering.id
         )
+        cls._cleanup.append(cls.vm_2)
         return
 
     @classmethod
@@ -1807,34 +1812,39 @@ class TestDomainForceRemove(cloudstackTestCase):
         #    not return any routers in the deleted accounts/domains
 
         self.debug("Creating a domain for login with API domain test")
-        domain = Domain.create(
+        self.child_domain = Domain.create(
             self.apiclient,
             self.services["domain"],
             parentdomainid=self.domain.id
         )
+        self.cleanup.append(self.child_domain)
         self.debug("Domain is created succesfully.")
         self.debug(
             "Checking if the created domain is listed in list domains API")
-        domains = Domain.list(self.apiclient, id=domain.id, listall=True)
+        domains = Domain.list(self.apiclient, id=self.child_domain.id, listall=True)
 
         self.assertEqual(
             isinstance(domains, list),
             True,
             "List domains shall return a valid response"
         )
-        self.debug("Creating 2 user accounts in domain: %s" % domain.name)
+        self.debug("Creating 2 user accounts in domain: %s" % self.child_domain.name)
         self.account_1 = Account.create(
             self.apiclient,
             self.services["account"],
-            domainid=domain.id
+            domainid=self.child_domain.id
         )
+        self.cleanup.append(self.account_1)
 
         self.account_2 = Account.create(
             self.apiclient,
             self.services["account"],
-            domainid=domain.id
+            domainid=self.child_domain.id
         )
+        self.cleanup.append(self.account_2)
 
+        vm_1 = None
+        vm_2 = None
         try:
             self.debug("Creating a tiny service offering for VM deployment")
             self.service_offering = ServiceOffering.create(
@@ -1842,10 +1852,11 @@ class TestDomainForceRemove(cloudstackTestCase):
                 self.services["service_offering"],
                 domainid=self.domain.id
             )
+            self.cleanup.append(self.service_offering)
 
             self.debug("Deploying virtual machine in account 1: %s" %
                        self.account_1.name)
-            vm_1 = VirtualMachine.create(
+            self.vm_1 = VirtualMachine.create(
                 self.apiclient,
                 self.services["virtual_machine"],
                 templateid=self.template.id,
@@ -1853,10 +1864,11 @@ class TestDomainForceRemove(cloudstackTestCase):
                 domainid=self.account_1.domainid,
                 serviceofferingid=self.service_offering.id
             )
+            self.cleanup.append(self.vm_1)
 
             self.debug("Deploying virtual machine in account 2: %s" %
                        self.account_2.name)
-            VirtualMachine.create(
+            self.vm_2 = VirtualMachine.create(
                 self.apiclient,
                 self.services["virtual_machine"],
                 templateid=self.template.id,
@@ -1864,6 +1876,7 @@ class TestDomainForceRemove(cloudstackTestCase):
                 domainid=self.account_2.domainid,
                 serviceofferingid=self.service_offering.id
             )
+            self.cleanup.append(self.vm_2)
 
             networks = Network.list(
                 self.apiclient,
@@ -1905,15 +1918,16 @@ class TestDomainForceRemove(cloudstackTestCase):
                 "Trying to create a port forwarding rule in source NAT: %s" %
                 src_nat.ipaddress)
             # Create NAT rule
-            nat_rule = NATRule.create(
+            self.nat_rule = NATRule.create(
                 self.apiclient,
-                vm_1,
+                self.vm_1,
                 self.services["natrule"],
                 ipaddressid=src_nat.id
             )
+            self.cleanup.append(self.nat_rule)
             self.debug("Created PF rule on source NAT: %s" % src_nat.ipaddress)
 
-            nat_rules = NATRule.list(self.apiclient, id=nat_rule.id)
+            nat_rules = NATRule.list(self.apiclient, id=self.nat_rule.id)
 
             self.assertEqual(
                 isinstance(nat_rules, list),
@@ -1927,15 +1941,18 @@ class TestDomainForceRemove(cloudstackTestCase):
                 "Length of response from listLbRules should not be 0"
             )
         except Exception as e:
-            self.cleanup.append(self.domain)
-            self.cleanup.append(self.account_1)
-            self.cleanup.append(self.account_2)
-            self.cleanup.append(self.service_offering)
             self.fail(e)
 
         self.debug("Deleting domain with force option")
         try:
-            domain.delete(self.apiclient, cleanup=True)
+            self.child_domain.delete(self.apiclient, cleanup=True)
+            self.cleanup.remove(self.child_domain)
+            self.cleanup.remove(self.account_1)
+            self.cleanup.remove(self.account_2)
+            self.cleanup.remove(self.vm_1)
+            self.cleanup.remove(self.vm_2)
+            self.cleanup.remove(self.nat_rule)
+
         except Exception as e:
             self.debug("Waiting for account.cleanup.interval" +
                        " to cleanup any remaining resouces")
@@ -1944,7 +1961,7 @@ class TestDomainForceRemove(cloudstackTestCase):
             with self.assertRaises(CloudstackAPIException):
                 Domain.list(
                     self.apiclient,
-                    id=domain.id,
+                    id=self.child_domain.id,
                     listall=True
                 )
 
@@ -1983,35 +2000,35 @@ class TestDomainForceRemove(cloudstackTestCase):
         # 5. domain deletion should fail saying there are resources under use
 
         self.debug("Creating a domain for login with API domain test")
-        domain = Domain.create(
+        self.child_domain = Domain.create(
             self.apiclient,
             self.services["domain"],
             parentdomainid=self.domain.id
         )
         # in this test delete domain *should* fail so we need to housekeep:
-        self.cleanup.append(domain)
-        self.debug("Domain: %s is created successfully." % domain.name)
+        self.cleanup.append(self.child_domain)
+        self.debug("Domain: %s is created successfully." % self.child_domain.name)
         self.debug(
             "Checking if the created domain is listed in list domains API")
-        domains = Domain.list(self.apiclient, id=domain.id, listall=True)
+        domains = Domain.list(self.apiclient, id=self.child_domain.id, listall=True)
 
         self.assertEqual(
             isinstance(domains, list),
             True,
             "List domains shall return a valid response"
         )
-        self.debug("Creating 2 user accounts in domain: %s" % domain.name)
+        self.debug("Creating 2 user accounts in domain: %s" % self.child_domain.name)
         self.account_1 = Account.create(
             self.apiclient,
             self.services["account"],
-            domainid=domain.id
+            domainid=self.child_domain.id
         )
         self.cleanup.append(self.account_1)
 
         self.account_2 = Account.create(
             self.apiclient,
             self.services["account"],
-            domainid=domain.id
+            domainid=self.child_domain.id
         )
         self.cleanup.append(self.account_2)
 
@@ -2025,7 +2042,7 @@ class TestDomainForceRemove(cloudstackTestCase):
 
         self.debug("Deploying virtual machine in account 1: %s" %
                    self.account_1.name)
-        vm_1 = VirtualMachine.create(
+        self.vm_1 = VirtualMachine.create(
             self.apiclient,
             self.services["virtual_machine"],
             templateid=self.template.id,
@@ -2033,10 +2050,11 @@ class TestDomainForceRemove(cloudstackTestCase):
             domainid=self.account_1.domainid,
             serviceofferingid=self.service_offering.id
         )
+        self.cleanup.append(self.vm_1)
 
         self.debug("Deploying virtual machine in account 2: %s" %
                    self.account_2.name)
-        VirtualMachine.create(
+        self.vm_2 = VirtualMachine.create(
             self.apiclient,
             self.services["virtual_machine"],
             templateid=self.template.id,
@@ -2044,6 +2062,7 @@ class TestDomainForceRemove(cloudstackTestCase):
             domainid=self.account_2.domainid,
             serviceofferingid=self.service_offering.id
         )
+        self.cleanup.append(self.vm_2)
 
         networks = Network.list(
             self.apiclient,
@@ -2085,15 +2104,16 @@ class TestDomainForceRemove(cloudstackTestCase):
             "Trying to create a port forwarding rule in source NAT: %s" %
             src_nat.ipaddress)
         # Create NAT rule
-        nat_rule = NATRule.create(
+        self.nat_rule = NATRule.create(
             self.apiclient,
-            vm_1,
+            self.vm_1,
             self.services["natrule"],
             ipaddressid=src_nat.id
         )
+        self.cleanup.append(self.nat_rule)
         self.debug("Created PF rule on source NAT: %s" % src_nat.ipaddress)
 
-        nat_rules = NATRule.list(self.apiclient, id=nat_rule.id)
+        nat_rules = NATRule.list(self.apiclient, id=self.nat_rule.id)
 
         self.assertEqual(
             isinstance(nat_rules, list),
@@ -2108,8 +2128,17 @@ class TestDomainForceRemove(cloudstackTestCase):
         )
 
         self.debug("Deleting domain without force option")
-        with self.assertRaises(Exception):
-            domain.delete(self.apiclient, cleanup=False)
+        try:
+            self.child_domain.delete(self.apiclient, cleanup=False)
+            self.cleanup.remove(self.nat_rule)
+            self.cleanup.remove(self.vm_2)
+            self.cleanup.remove(self.vm_1)
+            self.cleanup.remove(self.account_2)
+            self.cleanup.remove(self.account_1)
+            self.cleanup.remove(self.child_domain)
+            self.fail("shouldn't be able to delete domain")
+        except:
+            pass
         return
 
 class TestMoveUser(cloudstackTestCase):
@@ -2163,6 +2192,7 @@ class TestMoveUser(cloudstackTestCase):
             account=self.account1.name,
             domainid=self.account1.domainid
         )
+        self.cleanup.append(self.user)
 
         return
 
