@@ -368,8 +368,8 @@ def get_test_template(apiclient, zone_id=None, hypervisor=None, test_templates=N
     if hypervisor == 'simulator':
         return get_template(apiclient, zone_id)
 
-    if hypervisor not in test_templates.keys():
-        print "Provided hypervisor has no test template"
+    if hypervisor not in list(test_templates.keys()):
+        print("Provided hypervisor has no test template")
         return FAILED
 
     test_template = test_templates[hypervisor]
@@ -729,8 +729,8 @@ def list_os_types(apiclient, **kwargs):
     """List all os types matching criteria"""
 
     cmd = listOsTypes.listOsTypesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listOsTypes(cmd))
 
@@ -739,8 +739,8 @@ def list_routers(apiclient, **kwargs):
     """List all Routers matching criteria"""
 
     cmd = listRouters.listRoutersCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listRouters(cmd))
 
@@ -749,8 +749,8 @@ def list_zones(apiclient, **kwargs):
     """List all Zones matching criteria"""
 
     cmd = listZones.listZonesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listZones(cmd))
 
@@ -759,8 +759,8 @@ def list_networks(apiclient, **kwargs):
     """List all Networks matching criteria"""
 
     cmd = listNetworks.listNetworksCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listNetworks(cmd))
 
@@ -769,8 +769,8 @@ def list_clusters(apiclient, **kwargs):
     """List all Clusters matching criteria"""
 
     cmd = listClusters.listClustersCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listClusters(cmd))
 
@@ -779,8 +779,8 @@ def list_ssvms(apiclient, **kwargs):
     """List all SSVMs matching criteria"""
 
     cmd = listSystemVms.listSystemVmsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listSystemVms(cmd))
 
@@ -789,8 +789,8 @@ def list_storage_pools(apiclient, **kwargs):
     """List all storage pools matching criteria"""
 
     cmd = listStoragePools.listStoragePoolsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listStoragePools(cmd))
 
@@ -799,8 +799,8 @@ def list_virtual_machines(apiclient, **kwargs):
     """List all VMs matching criteria"""
 
     cmd = listVirtualMachines.listVirtualMachinesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listVirtualMachines(cmd))
 
@@ -809,8 +809,8 @@ def list_hosts(apiclient, **kwargs):
     """List all Hosts matching criteria"""
 
     cmd = listHosts.listHostsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listHosts(cmd))
 
@@ -819,8 +819,8 @@ def list_configurations(apiclient, **kwargs):
     """List configuration with specified name"""
 
     cmd = listConfigurations.listConfigurationsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listConfigurations(cmd))
 
@@ -829,8 +829,8 @@ def list_publicIP(apiclient, **kwargs):
     """List all Public IPs matching criteria"""
 
     cmd = listPublicIpAddresses.listPublicIpAddressesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listPublicIpAddresses(cmd))
 
@@ -839,8 +839,8 @@ def list_nat_rules(apiclient, **kwargs):
     """List all NAT rules matching criteria"""
 
     cmd = listPortForwardingRules.listPortForwardingRulesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listPortForwardingRules(cmd))
 
@@ -849,8 +849,8 @@ def list_lb_rules(apiclient, **kwargs):
     """List all Load balancing rules matching criteria"""
 
     cmd = listLoadBalancerRules.listLoadBalancerRulesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listLoadBalancerRules(cmd))
 
@@ -859,8 +859,8 @@ def list_lb_instances(apiclient, **kwargs):
     """List all Load balancing instances matching criteria"""
 
     cmd = listLoadBalancerRuleInstances.listLoadBalancerRuleInstancesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listLoadBalancerRuleInstances(cmd))
 
@@ -869,8 +869,8 @@ def list_firewall_rules(apiclient, **kwargs):
     """List all Firewall Rules matching criteria"""
 
     cmd = listFirewallRules.listFirewallRulesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listFirewallRules(cmd))
 
@@ -879,8 +879,8 @@ def list_volumes(apiclient, **kwargs):
     """List all volumes matching criteria"""
 
     cmd = listVolumes.listVolumesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listVolumes(cmd))
 
@@ -889,8 +889,8 @@ def list_isos(apiclient, **kwargs):
     """Lists all available ISO files."""
 
     cmd = listIsos.listIsosCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listIsos(cmd))
 
@@ -899,8 +899,8 @@ def list_snapshots(apiclient, **kwargs):
     """List all snapshots matching criteria"""
 
     cmd = listSnapshots.listSnapshotsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listSnapshots(cmd))
 
@@ -909,8 +909,8 @@ def list_templates(apiclient, **kwargs):
     """List all templates matching criteria"""
 
     cmd = listTemplates.listTemplatesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listTemplates(cmd))
 
@@ -919,8 +919,8 @@ def list_domains(apiclient, **kwargs):
     """Lists domains"""
 
     cmd = listDomains.listDomainsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listDomains(cmd))
 
@@ -930,8 +930,8 @@ def list_accounts(apiclient, **kwargs):
     listed accounts"""
 
     cmd = listAccounts.listAccountsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listAccounts(cmd))
 
@@ -941,8 +941,8 @@ def list_users(apiclient, **kwargs):
     listed users"""
 
     cmd = listUsers.listUsersCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listUsers(cmd))
 
@@ -951,8 +951,8 @@ def list_snapshot_policy(apiclient, **kwargs):
     """Lists snapshot policies."""
 
     cmd = listSnapshotPolicies.listSnapshotPoliciesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listSnapshotPolicies(cmd))
 
@@ -961,8 +961,8 @@ def list_events(apiclient, **kwargs):
     """Lists events"""
 
     cmd = listEvents.listEventsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listEvents(cmd))
 
@@ -971,8 +971,8 @@ def list_disk_offering(apiclient, **kwargs):
     """Lists all available disk offerings."""
 
     cmd = listDiskOfferings.listDiskOfferingsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listDiskOfferings(cmd))
 
@@ -981,8 +981,8 @@ def list_service_offering(apiclient, **kwargs):
     """Lists all available service offerings."""
 
     cmd = listServiceOfferings.listServiceOfferingsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listServiceOfferings(cmd))
 
@@ -991,8 +991,8 @@ def list_vlan_ipranges(apiclient, **kwargs):
     """Lists all VLAN IP ranges."""
 
     cmd = listVlanIpRanges.listVlanIpRangesCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listVlanIpRanges(cmd))
 
@@ -1001,8 +1001,8 @@ def list_usage_records(apiclient, **kwargs):
     """Lists usage records for accounts"""
 
     cmd = listUsageRecords.listUsageRecordsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listUsageRecords(cmd))
 
@@ -1011,8 +1011,8 @@ def list_nw_service_prividers(apiclient, **kwargs):
     """Lists Network service providers"""
 
     cmd = listNetworkServiceProviders.listNetworkServiceProvidersCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listNetworkServiceProviders(cmd))
 
@@ -1021,8 +1021,8 @@ def list_virtual_router_elements(apiclient, **kwargs):
     """Lists Virtual Router elements"""
 
     cmd = listVirtualRouterElements.listVirtualRouterElementsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listVirtualRouterElements(cmd))
 
@@ -1031,8 +1031,8 @@ def list_network_offerings(apiclient, **kwargs):
     """Lists network offerings"""
 
     cmd = listNetworkOfferings.listNetworkOfferingsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listNetworkOfferings(cmd))
 
@@ -1041,8 +1041,8 @@ def list_resource_limits(apiclient, **kwargs):
     """Lists resource limits"""
 
     cmd = listResourceLimits.listResourceLimitsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listResourceLimits(cmd))
 
@@ -1051,8 +1051,8 @@ def list_vpc_offerings(apiclient, **kwargs):
     """ Lists VPC offerings """
 
     cmd = listVPCOfferings.listVPCOfferingsCmd()
-    [setattr(cmd, k, v) for k, v in kwargs.items()]
-    if 'account' in kwargs.keys() and 'domainid' in kwargs.keys():
+    [setattr(cmd, k, v) for k, v in list(kwargs.items())]
+    if 'account' in list(kwargs.keys()) and 'domainid' in list(kwargs.keys()):
         cmd.listall=True
     return(apiclient.listVPCOfferings(cmd))
 
@@ -1144,8 +1144,8 @@ def get_free_vlan(apiclient, zoneid):
     usedVlanIds = []
 
     if isinstance(networks, list) and len(networks) > 0:
-        usedVlanIds = [int(nw.vlan)
-                       for nw in networks if (nw.vlan and str(nw.vlan).lower() != "untagged")]
+        usedVlanIds = [int(nw.broadcasturi.split("//")[-1])
+                       for nw in networks if (nw.broadcasturi and nw.broadcasturi.lower() != "vlan://untagged")]
 
     ipranges = list_vlan_ipranges(apiclient, zoneid=zoneid)
     if isinstance(ipranges, list) and len(ipranges) > 0:
@@ -1476,6 +1476,7 @@ def matchResourceCount(apiclient, expectedCount, resourceType,
                               accountid=None, projectid=None):
     """Match the resource count of account/project with the expected
     resource count"""
+    expected = int(expectedCount)  # initialise as int to make sure floats passed are acceptable
     try:
         resourceholderlist = None
         if accountid:
@@ -1489,6 +1490,7 @@ def matchResourceCount(apiclient, expectedCount, resourceType,
             resourceCount = resourceholderlist[0].primarystoragetotal
         elif resourceType == RESOURCE_SECONDARY_STORAGE:
             resourceCount = resourceholderlist[0].secondarystoragetotal
+            expected = expectedCount  # as the exception, an original value is needed here (should be of type float)
         elif resourceType == RESOURCE_CPU:
             resourceCount = resourceholderlist[0].cputotal
         elif resourceType == RESOURCE_MEMORY:
@@ -1507,7 +1509,7 @@ def matchResourceCount(apiclient, expectedCount, resourceType,
             resourceCount = resourceholderlist[0].networktotal
         elif resourceType == RESOURCE_VPC:
             resourceCount = resourceholderlist[0].vpctotal
-        assert str(resourceCount) == str(expectedCount),\
+        assert str(resourceCount) == str(expected),\
                 "Resource count %s should match with the expected resource count %s" %\
                 (resourceCount, expectedCount)
     except Exception as e:
@@ -2084,7 +2086,7 @@ def verifyVCenterPortGroups(
                 dvPortGroups = response[1]
                 expectedDVPortGroupNames.extend(dvPortGroups)
 
-        vcenterPortGroups = list(itertools.chain(*(switchDict.values())))
+        vcenterPortGroups = list(itertools.chain(*(list(switchDict.values()))))
 
         for expectedDVPortGroupName in expectedDVPortGroupNames:
             assert expectedDVPortGroupName in vcenterPortGroups,\
