@@ -27,7 +27,7 @@ except ImportError:
         raise RuntimeError("python setuptools is required to build Marvin")
 
 
-VERSION = "4.15.2.0"
+VERSION = "4.16.0.0"
 
 setup(name="Marvin",
       version=VERSION,
@@ -57,7 +57,10 @@ setup(name="Marvin",
           "ipmisim >= 0.7",
           "pytz",
           "retries",
-          "PyCrypt"
+          "PyCrypt",
+          "kubernetes",
+          "urllib3",
+          "setuptools >= 40.3.0"
       ],
       py_modules=['marvin.marvinPlugin'],
       zip_safe=False,
