@@ -23,7 +23,7 @@ export default {
   permission: ['listDomains', 'listDomainChildren'],
   resourceType: 'Domain',
   columns: ['name', 'state', 'path', 'parentdomainname', 'level'],
-  details: ['name', 'id', 'path', 'parentdomainname', 'level', 'networkdomain'],
+  details: ['name', 'id', 'path', 'parentdomainname', 'level', 'networkdomain', 'created'],
   component: () => import('@/views/iam/DomainView.vue'),
   related: [{
     name: 'account',
@@ -54,6 +54,9 @@ export default {
       name: 'settings',
       component: () => import('@/components/view/SettingsTab.vue'),
       show: (record, route, user) => { return ['Admin'].includes(user.roletype) }
+    }, {
+      name: 'comments',
+      component: () => import('@/components/view/AnnotationsTab.vue')
     }
   ],
   treeView: true,
