@@ -53,6 +53,7 @@ export default {
     title: 'label.secondary.storage',
     param: 'zoneid'
   }],
+  resourceType: 'Zone',
   tabs: [{
     name: 'details',
     component: () => import('@/components/view/DetailsTab.vue')
@@ -68,6 +69,9 @@ export default {
   }, {
     name: 'settings',
     component: () => import('@/components/view/SettingsTab.vue')
+  }, {
+    name: 'comments',
+    component: () => import('@/components/view/AnnotationsTab.vue')
   }],
   actions: [
     {
