@@ -68,6 +68,7 @@ import {
   AutoComplete,
   Collapse
 } from 'ant-design-vue'
+import VueCropper from 'vue-cropper'
 
 Vue.use(ConfigProvider)
 Vue.use(Layout)
@@ -117,6 +118,7 @@ Vue.use(Calendar)
 Vue.use(Slider)
 Vue.use(AutoComplete)
 Vue.use(Collapse)
+Vue.use(VueCropper)
 
 Vue.prototype.$confirm = Modal.confirm
 Vue.prototype.$message = message
