@@ -39,6 +39,8 @@ export const pollJobPlugin = {
        * @param {String} [catchMessage=Error caught]
        * @param {Function} [catchMethod=() => {}]
        * @param {Object} [action=null]
+       * @param {Object} [bulkAction=false]
+       * @param {String} resourceId
        */
       const {
         jobId,
@@ -53,7 +55,9 @@ export const pollJobPlugin = {
         showLoading = true,
         catchMessage = i18n.t('label.error.caught'),
         catchMethod = () => {},
-        action = null
+        action = null,
+        bulkAction = false,
+        resourceId = null
       } = options
 
       store.dispatch('AddHeaderNotice', {
@@ -63,9 +67,26 @@ export const pollJobPlugin = {
         status: 'progress'
       })
 
+      eventBus.$on('update-job-details', (jobId, resourceId) => {
+        const fullPath = this.$route.fullPath
+        const path = this.$route.path
+        var jobs = this.$store.getters.headerNotices.map(job => {
+          if (job.key === jobId) {
+            if (resourceId && !path.includes(resourceId)) {
+              job.path = path + '/' + resourceId
+            } else {
+              job.path = fullPath
+            }
+          }
+          return job
+        })
+        this.$store.commit('SET_HEADER_NOTICES', jobs)
+      })
+
       options.originalPage = options.originalPage || this.$router.currentRoute.path
       api('queryAsyncJobResult', { jobId }).then(json => {
         const result = json.queryasyncjobresultresponse
+        eventBus.$emit('update-job-details', jobId, resourceId)
         if (result.jobstatus === 1) {
           var content = successMessage
           if (successMessage === 'Success' && action && action.label) {
@@ -86,7 +107,7 @@ export const pollJobPlugin = {
             status: 'done',
             duration: 2
           })
-
+          eventBus.$emit('update-job-details', jobId, resourceId)
           // Ensure we refresh on the same / parent page
           const currentPage = this.$router.currentRoute.path
           const samePage = options.originalPage === currentPage || options.originalPage.startsWith(currentPage + '/')
@@ -95,33 +116,37 @@ export const pollJobPlugin = {
           }
           successMethod(result)
         } else if (result.jobstatus === 2) {
-          message.error({
-            content: errorMessage,
-            key: jobId,
-            duration: 1
-          })
-          var errTitle = errorMessage
+          if (!bulkAction) {
+            message.error({
+              content: errorMessage,
+              key: jobId,
+              duration: 1
+            })
+          }
+          var errMessage = errorMessage
           if (action && action.label) {
-            errTitle = i18n.t(action.label)
+            errMessage = i18n.t(action.label)
           }
           var desc = result.jobresult.errortext
           if (name) {
             desc = `(${name}) ${desc}`
           }
-          notification.error({
-            message: errTitle,
-            description: desc,
-            key: jobId,
-            duration: 0
-          })
+          if (!bulkAction) {
+            notification.error({
+              message: errMessage,
+              description: desc,
+              key: jobId,
+              duration: 0
+            })
+          }
           store.dispatch('AddHeaderNotice', {
             key: jobId,
             title: title,
             description: desc,
             status: 'failed',
-            duration: 0
+            duration: 2
           })
-
+          eventBus.$emit('update-job-details', jobId, resourceId)
           // Ensure we refresh on the same / parent page
           const currentPage = this.$router.currentRoute.path
           const samePage = options.originalPage === currentPage || options.originalPage.startsWith(currentPage + '/')
@@ -225,6 +250,39 @@ export const configUtilPlugin = {
   }
 }
 
+export const showIconPlugin = {
+  install (Vue) {
+    Vue.prototype.$showIcon = function (resource) {
+      var resourceType = this.$route.path.split('/')[1]
+      if (resource) {
+        resourceType = resource
+      }
+      if (['zone', 'template', 'iso', 'account', 'accountuser', 'vm', 'domain', 'project', 'vpc', 'guestnetwork'].includes(resourceType)) {
+        return true
+      } else {
+        return false
+      }
+    }
+  }
+}
+
+export const resourceTypePlugin = {
+  install (Vue) {
+    Vue.prototype.$getResourceType = function () {
+      const type = this.$route.path.split('/')[1]
+      if (type === 'vm') {
+        return 'UserVM'
+      } else if (type === 'accountuser') {
+        return 'User'
+      } else if (type === 'guestnetwork') {
+        return 'Network'
+      } else {
+        return type
+      }
+    }
+  }
+}
+
 export const apiMetaUtilPlugin = {
   install (Vue) {
     Vue.prototype.$getApiParams = function () {
