@@ -22,7 +22,7 @@ import mockData from '../../../mockData/MigrateWizard.mock'
 
 jest.mock('axios', () => mockAxios)
 
-let wrapper, i18n, store, mocks
+let wrapper, i18n, store, mocks, router
 
 const state = {}
 mocks = {
@@ -59,8 +59,10 @@ const factory = (opts = {}) => {
   i18n = opts.i18n || i18n
   store = opts.store || store
   mocks = opts.mocks || mocks
+  router = opts.router || router
 
   return common.createFactory(MigrateWizard, {
+    router,
     i18n,
     store,
     mocks,
@@ -70,7 +72,7 @@ const factory = (opts = {}) => {
 }
 
 describe('Views > compute > MigrateWizard.vue', () => {
-  jest.spyOn(console, 'warn').mockImplementation(() => {})
+  // jest.spyOn(console, 'warn').mockImplementation(() => {})
 
   beforeEach(() => {
     jest.clearAllMocks()
@@ -78,6 +80,9 @@ describe('Views > compute > MigrateWizard.vue', () => {
     if (wrapper) {
       wrapper.destroy()
     }
+    if (router && router.currentRoute.name !== 'home') {
+      router.replace({ name: 'home' })
+    }
 
     if (i18n.locale !== 'en') {
       i18n.locale = 'en'
@@ -328,7 +333,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
     })
 
     describe('submitForm()', () => {
-      it('check api is called when selectedHost.requiresStorageMotion is true', async (done) => {
+      it('check api is called when selectedHost.requiresStorageMotion is true and isUserVm=true', async (done) => {
         const mockData = {
           migratevirtualmachineresponse: {
             jobid: 'test-job-id'
@@ -340,8 +345,17 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         }
+        mockAxios.mockResolvedValue(mockData)
 
+        router = common.createMockRouter([{
+          name: 'testRouter1',
+          path: '/test-router-1',
+          meta: {
+            name: 'vm'
+          }
+        }])
         wrapper = factory({
+          router: router,
           props: {
             resource: {
               id: 'test-resource-id',
@@ -356,8 +370,8 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-        mockAxios.mockResolvedValue(mockData)
+        router.push({ name: 'testRouter1' })
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -379,7 +393,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
         })
       })
 
-      it('check api is called when selectedHost.requiresStorageMotion is false', async (done) => {
+      it('check api is called when selectedHost.requiresStorageMotion is false and isUserVm=true', async (done) => {
         const mockData = {
           migratevirtualmachineresponse: {
             jobid: 'test-job-id'
@@ -391,6 +405,15 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         }
+        mockAxios.mockResolvedValue(mockData)
+
+        router = common.createMockRouter([{
+          name: 'testRouter2',
+          path: '/test-router-2',
+          meta: {
+            name: 'vm'
+          }
+        }])
         wrapper = factory({
           props: {
             resource: {
@@ -406,10 +429,9 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-
-        mockAxios.mockResolvedValue(mockData)
 
+        router.push({ name: 'testRouter2' })
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -431,6 +453,65 @@ describe('Views > compute > MigrateWizard.vue', () => {
         })
       })
 
+      it('check api is called when isUserVm=false', async (done) => {
+        const mockData = {
+          migratevirtualmachineresponse: {
+            jobid: 'test-job-id'
+          },
+          queryasyncjobresultresponse: {
+            jobstatus: 1,
+            jobresult: {
+              name: 'test-name-value'
+            }
+          }
+        }
+        mockAxios.mockResolvedValue(mockData)
+        router = common.createMockRouter([{
+          name: 'testRouter3',
+          path: '/test-router-3',
+          meta: {
+            name: 'test'
+          }
+        }])
+        wrapper = factory({
+          router: router,
+          props: {
+            resource: {
+              id: 'test-resource-id',
+              name: 'test-resource-name'
+            }
+          },
+          data: {
+            selectedHost: {
+              requiresStorageMotion: true,
+              id: 'test-host-id',
+              name: 'test-host-name'
+            }
+          }
+        })
+        router.push({ name: 'testRouter3' })
+        wrapper.vm.loading = false
+        await wrapper.vm.$nextTick()
+        await wrapper.vm.submitForm()
+
+        setTimeout(() => {
+          expect(mockAxios).toHaveBeenCalled()
+          expect(mockAxios).toHaveBeenCalledWith({
+            url: '/',
+            method: 'GET',
+            data: new URLSearchParams(),
+            params: {
+              command: 'migrateSystemVm',
+              hostid: 'test-host-id',
+              virtualmachineid: 'test-resource-id',
+              response: 'json'
+            }
+          })
+
+          done()
+        })
+      })
+
       it('check $pollJob have successMethod() is called with requiresStorageMotion is true', async (done) => {
         const mockData = {
           migratevirtualmachinewithvolumeresponse: {
@@ -443,7 +524,16 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         }
+        mockAxios.mockResolvedValue(mockData)
+        router = common.createMockRouter([{
+          name: 'testRouter4',
+          path: '/test-router-4',
+          meta: {
+            name: 'vm'
+          }
+        }])
         wrapper = factory({
+          router: router,
           props: {
             resource: {
               id: 'test-resource-id',
@@ -458,10 +548,8 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-
-        mockAxios.mockResolvedValue(mockData)
-
+        router.push({ name: 'testRouter4' })
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -485,7 +573,16 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         }
+        mockAxios.mockResolvedValue(mockData)
+        router = common.createMockRouter([{
+          name: 'testRouter5',
+          path: '/test-router-5',
+          meta: {
+            name: 'vm'
+          }
+        }])
         wrapper = factory({
+          router: router,
           props: {
             resource: {
               id: 'test-resource-id',
@@ -500,10 +597,56 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
+        router.push({ name: 'testRouter5' })
+        wrapper.vm.loading = false
+        await wrapper.vm.$nextTick()
+        await wrapper.vm.submitForm()
 
+        setTimeout(() => {
+          expect(mocks.$pollJob).toHaveBeenCalled()
+          expect(wrapper.emitted()['close-action'][0]).toEqual([])
+          done()
+        })
+      })
+
+      it('check $pollJob have successMethod() is called with isUserVm is false', async (done) => {
+        const mockData = {
+          migratesystemvmresponse: {
+            jobid: 'test-job-id-case-2'
+          },
+          queryasyncjobresultresponse: {
+            jobstatus: 1,
+            jobresult: {
+              name: 'test-name-value'
+            }
+          }
+        }
         mockAxios.mockResolvedValue(mockData)
-
+        router = common.createMockRouter([{
+          name: 'testRouter6',
+          path: '/test-router-6',
+          meta: {
+            name: 'test'
+          }
+        }])
+        wrapper = factory({
+          router: router,
+          props: {
+            resource: {
+              id: 'test-resource-id',
+              name: 'test-resource-name'
+            }
+          },
+          data: {
+            selectedHost: {
+              requiresStorageMotion: false,
+              id: 'test-host-id',
+              name: 'test-host-name'
+            }
+          }
+        })
+        router.push({ name: 'testRouter6' })
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -517,7 +660,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
 
       it('check $pollJob have errorMethod() is called', async (done) => {
         const mockData = {
-          migratevirtualmachinewithvolumeresponse: {
+          migratesystemvmresponse: {
             jobid: 'test-job-id-case-3'
           },
           queryasyncjobresultresponse: {
@@ -527,6 +670,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         }
+        mockAxios.mockResolvedValue(mockData)
         wrapper = factory({
           props: {
             resource: {
@@ -542,10 +686,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-
-        mockAxios.mockResolvedValue(mockData)
-
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -559,10 +700,11 @@ describe('Views > compute > MigrateWizard.vue', () => {
 
       it('check $pollJob have catchMethod() is called', async (done) => {
         const mockData = {
-          migratevirtualmachinewithvolumeresponse: {
+          migratesystemvmresponse: {
             jobid: 'test-job-id-case-4'
           }
         }
+        mockAxios.mockResolvedValue(mockData)
         wrapper = factory({
           props: {
             resource: {
@@ -578,10 +720,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-
-        mockAxios.mockResolvedValue(mockData)
-
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
@@ -597,7 +736,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
         const mockError = {
           message: 'Error: throw error message'
         }
-
+        mockAxios.mockRejectedValue(mockError)
         wrapper = factory({
           props: {
             resource: {}
@@ -610,10 +749,7 @@ describe('Views > compute > MigrateWizard.vue', () => {
             }
           }
         })
-        jest.spyOn(wrapper.vm, 'fetchData').mockImplementation(() => {})
-
-        mockAxios.mockRejectedValue(mockError)
-
+        wrapper.vm.loading = false
         await wrapper.vm.$nextTick()
         await wrapper.vm.submitForm()
 
